import { FollowMode } from './follow-mode';
export interface FingerprintOptions {
    /**
     * Extra information to encode into the fingerprint (e.g. build instructions
     * and other inputs)
     */
    extra?: string;
    /**
     * List of exclude patterns (see `CopyOptions`)
     * @default include all files
     */
    exclude?: string[];
    /**
     * What to do when we encounter symlinks.
     * @default External only follows symlinks that are external to the source
     * directory
     */
    follow?: FollowMode;
}
/**
 * Produces fingerprint based on the contents of a single file or an entire directory tree.
 *
 * The fingerprint will also include:
 * 1. An extra string if defined in `options.extra`.
 * 2. The set of exclude patterns, if defined in `options.exclude`
 * 3. The symlink follow mode value.
 *
 * @param fileOrDirectory The directory or file to fingerprint
 * @param options Fingerprinting options
 */
export declare function fingerprint(fileOrDirectory: string, options?: FingerprintOptions): string;
